# (c) 2025 ANSYS, Inc. Unauthorized use, distribution, or duplication is prohibited.
"""Autogenerated Python gRPC interface package for ansys-api-speos."""

__all__ = ["__version__"]

from ansys.api.speos._version import __version__  # noqa: F401