// (c) 2025 ANSYS, Inc. Unauthorized use, distribution, or duplication is prohibited.
syntax = "proto3";

import "ansys/api/speos/sensor/v1/common.proto";

package ansys.api.speos.sensor.v1;

message IrradianceSensorTemplate {

	oneof sensor_type {
		ansys.api.speos.sensor.v1.SensorTypePhotometric sensor_type_photometric = 2; // Sensor type : Photometric
		ansys.api.speos.sensor.v1.SensorTypeColorimetric sensor_type_colorimetric = 3; //  Sensor type : Colorimetric
		ansys.api.speos.sensor.v1.SensorTypeRadiometric sensor_type_radiometric = 4; //  Sensor type : Radiometric
		ansys.api.speos.sensor.v1.SensorTypeSpectral sensor_type_spectral = 5; // Sensor type : Spectral
	}
	
	// Select how the light should be integrated into the sensor.
	oneof illuminance_type {
		ansys.api.speos.sensor.v1.IlluminanceTypePlanar illuminance_type_planar = 6; // Illuminance type : Planar
		ansys.api.speos.sensor.v1.IlluminanceTypeRadial illuminance_type_radial = 7; // Illuminance type : Radial
		ansys.api.speos.sensor.v1.IlluminanceTypeHemispherical illuminance_type_hemispherical = 8; // Illuminance type : Hemispherical
		ansys.api.speos.sensor.v1.IlluminanceTypeCylindrical illuminance_type_cylindrical = 9; // Illuminance type : Cylindrical
		ansys.api.speos.sensor.v1.IlluminanceTypeSemiCylindrical illuminance_type_semi_cylindrical = 10; // Illuminance type : SemiCylindrical
	}
	
	ansys.api.speos.sensor.v1.SensorDimensions dimensions = 11; // Dimensions of the sensor
}
